from django.contrib import admin
from django.contrib.staticfiles.storage import staticfiles_storage
from jet.admin import CompactInline


class JetTabularInline(admin.TabularInline):
    '''
        An override of the default Django TabularInline with some extra styles
        and some fixes for things that are broken about the Jet implementation.

        In this case, the js fixes:
            - Select2 gets initialised twice when you add a new field.

        In this case, the css fixes:
            - The hidden form is visible by default.
    '''

    class Media:
        js = [
            staticfiles_storage.url('admin/edit_inline/tabular_inline.js'),
        ]

        css = {
            'all': [
                staticfiles_storage.url('admin/edit_inline/tabular_inline.css'),
            ],
        }


class JetStackedInline(admin.StackedInline):
    '''
        An override of the default Django StackedInline with some extra styles
        and some fixes for things that are broken about the Jet implementation.

        In this case, the js fixes:
            - Select2 gets initialised twice when you add a new field.
            - If using the Onespacemedia CMS, the rich text fields do not get
              initialised on page load. (This is also an issue on TabularInlines
              however rich text fields should not be used in those)

        In this case, the css fixes:
            - The hidden form is visible by default.
    '''

    class Media:
        js = [
            staticfiles_storage.url('admin/edit_inline/stacked_inline.js'),
        ]

        css = {
            'all': [
                staticfiles_storage.url('admin/edit_inline/stacked_inline.css'),
            ],
        }


class JetCompactInline(CompactInline):
    '''
        An override of the default Jet CompactInline with some extra styles
        and some fixes for things that are broken about the Jet implementation
        along with some improvements.

        In this case, the js fixes:
            - An issue with RawIdFields that caused them to not be updated on
              new inlines when using the lookup.
            - An issue with ForeignKeyFields that caused the options to not get
              updated if a new object was added.
            - If using the Onespacemedia CMS, the rich text fields do not get
              initialised on page load.
        The js also adds:
            - The ability to order the CompactInline inlines.
            - Indicates clearly the incorrect inline in case of errors.

        In this case, the css adds:
            - Styles for the new re-ordering functionality.
    '''

    class Media:
        js = [
            staticfiles_storage.url('admin/sortable/sortables.min.js'),
        ]

        css = {
            'all': [
                staticfiles_storage.url('admin/edit_inline/compact_inline.css'),
            ],
        }



from cms.apps.pages.admin import (PAGE_FROM_KEY, PAGE_FROM_SITEMAP_VALUE,
                                  page_admin)
from cms.apps.pages.models import Page
from django.conf import settings
from django.urls import reverse
from jet.dashboard.modules import DashboardModule


def generate_sitemap(request):
    '''
        A function that takes a request and builds the sitemap for the current user from it.

        Returns a dictionary of the form with the following fields:
            canAdd:            True if the current user can add a page else False
            createHomepageUrl: The URL for adding the homepage. Used if no pages have been added yet
            moveUrl:           The AJAX POST URL used when moving items in the sitemap
            entries:           An array of dictionaries:

        The entries array contains dictionaries with the following fields:
            isOnline:        True if the page is online else False
            pageId:          The id of the page
            title:           The title of the page
            changeUrl:       The admin URL of the page
            siteUrl:         The site URL of the page
            children:        An array of dictionaries of the same form as this one for this page's children
            canChange:       If the user can edit this page.
            extra_languages: Any extra languages for the page. If there are any, this will be an array of dictionaries.

        The extra_languages array contains dictionaries with the following fields:
            changeURL: The admin URL of the page for this language
            language: The first country object for the language
    '''

    homepage = request.pages.homepage

    # Compile the initial data.
    data = {
        'canAdd': page_admin.has_add_permission(request),
        'createHomepageUrl': reverse('admin:pages_page_add') + '?{0}={1}'.format(PAGE_FROM_KEY, PAGE_FROM_SITEMAP_VALUE),
        'moveUrl': reverse('admin:pages_page_move_page') or None,
    }

    # Add in the page data.
    if homepage:
        def sitemap_entry(page):
            children = []
            for child in page.children:
                children.append(sitemap_entry(child))

            outcome_dict = {
                'isOnline': page.is_online,
                'pageId': page.id,
                'title': str(page),
                'changeUrl': reverse('admin:pages_page_change', args=[page.id]) + '?{0}={1}'.format(PAGE_FROM_KEY, PAGE_FROM_SITEMAP_VALUE),
                'siteUrl': '{}?preview=1'.format(page.get_absolute_url()),
                'children': children,
                'canChange': page_admin.has_change_permission(request, page),
                'extra_languages': []
            }

            middleware = settings.MIDDLEWARE or settings.MIDDLEWARE_CLASSES

            if not 'cms.middleware.LocalisationMiddleware' in middleware:
                return outcome_dict

            # Get all of the language pages
            extra_language_pages = Page.objects.filter(
                owner=page,
                is_content_object=True
            ).order_by('-country_group')

            outcome_dict['extra_languages'] = [{
                'changeURL': reverse('admin:pages_page_change', args=[page.id]) + '?{0}={1}'.format(PAGE_FROM_KEY, PAGE_FROM_SITEMAP_VALUE),
                'language': page.country_group.country_set.first()
            } for page in extra_language_pages]

            return outcome_dict

        data['entries'] = [sitemap_entry(homepage)]
    else:
        data['entries'] = []

    return data


class SiteMap(DashboardModule):
    '''
        The sitemap module for Onespacemedia CMS.

        Renders a page tree of the site for the users to allow quick navigation to
        editing pages and to compactly display page information.

        We don't want this to be deletable but draggable is fine.
    '''

    title = 'Sitemap'
    template = 'admin/dashboard_modules/sitemap.html'
    draggable = True
    deletable = True
    collapsible = False

    def get_context_data(self):
        '''
            We need to pass the sitemap information to the context so we can render
            it out in the template.
        '''
        context = super().get_context_data()
        context['sitemap_json'] = generate_sitemap(context['request'])

        return context



from collections import OrderedDict

from django import template
from django.urls import reverse
from django.utils.text import slugify
from jet import settings
from jet.models import PinnedApplication
from jet.utils import get_admin_site, get_original_menu_items

register = template.Library()
assignment_tag = register.assignment_tag if hasattr(register, 'assignment_tag') else register.simple_tag


def get_menu_item_url(url, original_app_list):
    if isinstance(url, dict):

        url_type = url.get('type')

        if not url_type == 'reverse' and not original_app_list.get(url['app_label']):
            return ''

        if url_type == 'app':
            return original_app_list[url['app_label']]['url']
        elif url_type == 'model':
            models = dict(map(
                lambda x: (x['name'], x['url']),
                original_app_list[url['app_label']]['models']
            ))
            return models[url['model']]
        elif url_type == 'reverse':
            return reverse(url['name'], args=url.get('args'), kwargs=url.get('kwargs'))
    elif isinstance(url, str):
        return url


# pylint: disable=too-complex
@assignment_tag(takes_context=True)
def osm_jet_get_menu(context):
    pinned_apps = PinnedApplication.objects.filter(user=context['user'].pk).values_list('app_label', flat=True)
    original_app_list = OrderedDict(map(lambda app: (app['app_label'], app), get_original_menu_items(context)))
    custom_app_list = settings.JET_SIDE_MENU_ITEMS
    custom_app_list_deprecated = settings.JET_SIDE_MENU_CUSTOM_APPS

    if custom_app_list not in (None, False):
        if isinstance(custom_app_list, dict):
            admin_site = get_admin_site(context)
            custom_app_list = custom_app_list.get(admin_site.name, [])

        app_list = []

        def get_menu_item_app_model(app_label, data):
            item = {'has_perms': True}

            if 'name' in data:
                parts = data['name'].split('.', 2)

                if len(parts) > 1:
                    app_label, name = parts
                else:
                    name = data['name']

                if app_label in original_app_list:
                    models = dict(map(
                        lambda x: (x['name'], x),
                        original_app_list[app_label]['models']
                    ))

                    if name in models:
                        item = models[name].copy()

            if 'label' in data:
                item['label'] = data['label']

            if 'url' in data:
                item['url'] = get_menu_item_url(data['url'], original_app_list)

            if 'url_blank' in data:
                item['url_blank'] = data['url_blank']

            if 'permissions' in data:
                item['has_perms'] = context['user'].has_perms(data['permissions'])

            return item

        def get_menu_item_app(data):
            app_label = data.get('app_label')

            if not app_label:
                if 'label' not in data:
                    raise Exception('Custom menu items should at least have \'label\' or \'app_label\' key')
                app_label = 'custom_%s' % slugify(data['label'], allow_unicode=True)

            if app_label in original_app_list:
                item = original_app_list[app_label].copy()
            else:
                item = {'app_label': app_label, 'has_perms': True}

            if 'label' in data:
                item['label'] = data['label']

            if 'items' in data:
                item['items'] = list(map(lambda x: get_menu_item_app_model(app_label, x), data['items']))
                item['items'] = [item for item in item['items'] if item.get('url')]

            if 'url' in data:
                item['url'] = get_menu_item_url(data['url'], original_app_list)

            if 'url_blank' in data:
                item['url_blank'] = data['url_blank']

            if 'permissions' in data:
                item['has_perms'] = item.get('has_perms', True) and context['user'].has_perms(data['permissions'])

            item['pinned'] = item['app_label'] in pinned_apps

            return item

        for data in custom_app_list:
            item = get_menu_item_app(data)
            app_list.append(item)
    elif custom_app_list_deprecated not in (None, False):
        app_dict = {}
        models_dict = {}

        for app in original_app_list.values():
            app_label = app['app_label']
            app_dict[app_label] = app

            for model in app['models']:
                if app_label not in models_dict:
                    models_dict[app_label] = {}

                models_dict[app_label][model['object_name']] = model

            app['items'] = []

        app_list = []

        if isinstance(custom_app_list_deprecated, dict):
            admin_site = get_admin_site(context)
            custom_app_list_deprecated = custom_app_list_deprecated.get(admin_site.name, [])

        for item in custom_app_list_deprecated:
            app_label, models = item

            if app_label in app_dict:
                app = app_dict[app_label]

                for model_label in models:
                    if model_label == '__all__':
                        app['items'] = models_dict[app_label].values()
                        break
                    elif model_label in models_dict[app_label]:
                        model = models_dict[app_label][model_label]
                        app['items'].append(model)

                app_list.append(app)
    else:
        def map_item(item):
            item['items'] = item['models']
            return item
        app_list = list(map(map_item, original_app_list.values()))

    current_found = False

    for app in app_list:
        if not current_found:
            for model in app['items']:
                if not current_found and model.get('url') and context['request'].path.startswith(model['url']):
                    model['current'] = True
                    current_found = True
                else:
                    model['current'] = False

            if not current_found and app.get('url') and context['request'].path.startswith(app['url']):
                app['current'] = True
                current_found = True
            else:
                app['current'] = False

    app_list = [app for app in app_list if app.get('items')]

    return app_list



from jet.dashboard.dashboard import Dashboard as JetDashboard

from .modules import SiteMap


class Dashboard(JetDashboard):
    '''
        An override of the default Jet dashboard. Sets the default number of
        columns to 3 and ensures the user always has a single Sitemap module
        on their dashboard.
    '''

    columns = 3

    def init_with_context(self, context):
        super().init_with_context(context)

        self.children.append(SiteMap(
            'Sitemap',
            column=2,
        ))



from itertools import chain

from django.contrib.staticfiles.storage import staticfiles_storage
from django.db import models
from django.forms import CheckboxInput, CheckboxSelectMultiple, SelectMultiple
from django.urls import NoReverseMatch, reverse
from django.utils.encoding import force_text
from django.utils.html import conditional_escape

# Catch the case that the user doesn't have sortedM2M installed
try:
    from sortedm2m.forms import SortedMultipleChoiceField
    from sortedm2m.fields import SortedManyToManyField
except ImportError:
    class SortedMultipleChoiceField:
        pass

    class SortedManyToManyField:
        pass


class DjangoManyToManyWidget(SelectMultiple):
    '''
        The widget for the old-style ManyToMany field.
    '''

    template_name = 'widgets/select_multiple.html'

    class Media:
        css = {
            'all': [
                staticfiles_storage.url('admin/widgets/many2many.css'),
            ],
        }

        js = [
            'admin/widgets/many2many.js'
        ]


class DjangoManyToMany(models.ManyToManyField):
    '''
        A custom ManyToMany field type to bring back the old Django style
        ManyToMany fields if you're not a fan of the select2 style ManyToMany
        fields.
    '''

    def formfield(self, **kwargs):  # pylint:disable=arguments-differ
        kwargs['widget'] = DjangoManyToManyWidget
        return super().formfield(**kwargs)


class SortedManyToManyWidget(CheckboxSelectMultiple):
    '''
        A customised version of the sortedM2M SortedCheckboxSelectMultiple.

        Adds an inline edit link for each object in the widget.

        Code has been refactored a bit from the base sortedM2M
        SortedCheckboxSelectMultipleto be more up to date with widget
        rendering styles.
    '''
    template_name = 'widgets/sortedm2m_editlinks.html'

    class Media:
        js = (
            'admin/js/jquery.init.js',
            'sortedm2m/widget.js',
            'sortedm2m/jquery-ui.js',
        )
        css = {'screen': (
            'sortedm2m/widget.css',
        )}

    def __init__(self, **kwargs):
        self.model_cls = None
        super().__init__(**kwargs)

    def build_attrs(self, base_attrs, extra_attrs=None):
        attrs = dict(base_attrs or {}, **extra_attrs)

        attrs = super().build_attrs(attrs)

        classes = attrs.setdefault('class', '').split()
        classes.append('sortedm2m')
        attrs['class'] = ' '.join(classes)

        return attrs

    def get_context(self, name, value, attrs):
        context = super().get_context(name, value, attrs)

        selected, unselected = self.get_selected_and_unselected(name, value, attrs)

        context['selected'] = selected
        context['unselected'] = unselected

        return context

    def value_from_datadict(self, data, files, name):
        value = data.get(name, None)

        if isinstance(value, str):
            return [v for v in value.split(',') if v]

        return value

    def get_selected_and_unselected(self, name, value, attrs):
        if value is None:
            value = []

        has_id = attrs and 'id' in attrs
        final_attrs = self.build_attrs(attrs, extra_attrs={'name': name})

        # Normalize to strings
        str_values = [force_text(v) for v in value]

        selected = []
        unselected = []

        for i, (option_value, option_label) in enumerate(chain(self.choices)):
            # If an ID attribute was given, add a numeric index as a suffix,
            # so that the checkboxes don't all have the same ID attribute.
            if has_id:
                final_attrs = dict(final_attrs, id='%s_%s' % (attrs['id'], i))
                label_for = ' for="%s"' % conditional_escape(final_attrs['id'])
            else:
                label_for = ''

            cb = CheckboxInput(final_attrs, check_test=lambda value: value in str_values)
            option_value = force_text(option_value)
            rendered_cb = cb.render(name, option_value)
            option_label = conditional_escape(force_text(option_label))

            try:
                edit_link = reverse(
                    f'admin:{self.model_cls._meta.app_label}_{self.model_cls._meta.model_name}_change',
                    args=[option_value]
                )
            except NoReverseMatch:
                edit_link = None

            item = {
                'label_for': label_for,
                'rendered_cb': rendered_cb,
                'option_label': option_label,
                'option_value': option_value,
                'edit_link': edit_link
            }

            if option_value in str_values:
                selected.append(item)
            else:
                unselected.append(item)

        # Reorder `selected` array according str_values which is a set of `option_value`s in the
        # order they should be shown on screen
        ordered = []

        for s in str_values:
            for select in selected:
                if s == select['option_value']:
                    ordered.append(select)

        selected = ordered

        return selected, unselected


class JetSortedMultipleChoiceField(SortedMultipleChoiceField):
    widget = SortedManyToManyWidget

    def __init__(self, **kwargs):
        self.model_cls = kwargs.pop('model_cls')
        super().__init__(**kwargs)
        self.widget.model_cls = self.model_cls

class JetSortedManyToManyField(SortedManyToManyField):
    def formfield(self, **kwargs):
        defaults = {}

        if self.sorted:
            defaults['form_class'] = JetSortedMultipleChoiceField
            defaults['model_cls'] = self.remote_field.model

        defaults.update(kwargs)

        return super().formfield(**defaults)



from django import template

register = template.Library()


@register.inclusion_tag('admin/submit_line.html', takes_context=True)
def save_buttons(context):
    '''
        An extension of the default Django save_buttons templatetag.

        We've moved the 'view on site' button to also be inline with the save buttons
        and have made 'Save and continue editing' the default save option. The other
        save options are made into a dropdown to avoid a large amount of clutter at the
        bottom of the page.
    '''
    opts = context['opts']
    change = context['change']
    is_popup = context['is_popup']
    save_as = context['save_as']
    original = context.get('original', False)
    return {
        'onclick_attrib': (change and 'onclick="submitOrderForm();"' or ''),
        'show_delete_link': (not is_popup and context['has_delete_permission'] and (change or context.get('show_delete', False))), # show_delete is not set on creation pages
        'show_save_as_new': not is_popup and change and save_as,
        'show_save_and_add_another': context['has_add_permission'] and not is_popup and (not save_as or context['add']),
        'show_save_and_continue': not is_popup and context['has_change_permission'],
        'is_popup': is_popup,
        'show_save': True,
        'opts': opts,
        'original': original,
        'model_name': original._meta.model_name if original else '',
    }



from distutils.core import setup

from setuptools import find_packages

setup(
    name='osm-jet',
    version='1.1.9',
    license='agpl-3.0',
    description='A CMS skin for Django. An extension of django-jet with some fixes and additional features',
    author='Onespacemedia',
    author_email='developers@onespacemedia.com',
    url='https://github.com/onespacemedia/osm_jet',
    packages=find_packages(),
    include_package_data=True,
    install_requires=[
        'Django',
        'django-jet',
    ],
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Framework :: Django :: 1.11',
        'License :: OSI Approved :: GNU Affero General Public License v3',
        'Natural Language :: English',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
    ],
)
